///////////////////////////////////////////////////////////////////
//                                                               //
//   This short program implements QuickSort on a list           //
//                                                               //
//   This is a basic implementation of quicksort on a doubly-    //
//   linked list. It is used to compare with ListSort, which     //
//   implements a radix sort (or distribution sort)              //
//                                                               //
//   (c) 2002 Steven Pigeon (pigeon@iro.umontreal.ca)            //
//                                                               //
///////////////////////////////////////////////////////////////////


#include <stdlib.h>
#include <stdio.h>
#include <time.h>

//////////////////////////////////////
// A simple doubly-linked list node //
//////////////////////////////////////
class cListNode
       {
        public:

         int number; // the number; could be anything else (''body'' of node)
         cListNode *next, *prev; // linking both ways

         cListNode(int i=0); // cheapo constructor to set ptrs to NULL and number to i (defaults to 0)
       };


//////////////////
// A list class //
//////////////////
class cList
       {
        public: 

         int count; // counts the number of elements

         cListNode *head,*tail; // pointers to head and tail of list

         void add(int); // adds a number at the end of the list

         cList();
         ~cList();
       };


////////////////////////////////
//                            //
// Basic list implementation. //
//                            //
////////////////////////////////

////////////////////////////////////////////////
cListNode::cListNode(int i)
 {
  prev=next=NULL;
  number=i;
 }


////////////////////////////////////////////////
void cList::add(int i)
 {
  cListNode *t = new cListNode(i);

  if (tail)
       {
        tail->next = t;
        t->prev = tail;
        tail = t;
       }
  else head=tail=t;

  count++;
 }

////////////////////////////////////////////////
cList::cList()
 {
  head=tail=NULL;
  count=0;
 }

////////////////////////////////////////////////
cList::~cList()
 {
  while (head)
   {
    cListNode *t = head;
    head = head->next;
    delete t;
   }
  count=0;
 }



//////////////////////////////////
void InitList(cList & t, int length)
 {
  while (length)
   {
    t.add(rand()*rand());
    length--;
   }
 }


//////////////////////////////////
void DisplayList(cList & t)
 {
  cListNode *current = t.head;

  while (current)
   {
    printf("%d\n", current->number);
    current = current->next;
   }
 }

////////////////////////////////////
template <typename T> void swap(T & x, T & y) { T t=x; x=y; y=t; }
                                                  
////////////////////////////////////
void QuickSortList(cListNode *head, cListNode *tail, int count)
 {
  cListNode * temp = head;
  for (int i=0;i<count/2;i++) temp=temp->next;

  const int pivot = temp->number;


  cListNode * left = head;
  cListNode * right = tail;

  int left_c = 0;
  int right_c = count-1;

  do
   {
    while ( left->number < pivot ) left=left->next, left_c++;

    while ( (right_c>=left_c) && right->number > pivot ) right=right->prev, right_c--; 

    if (left_c<=right_c) 
       {
        // exchanges contents only (same as swaping a data pointer)
        swap(left->number, right->number);

        left  = left->next;
        right = right->prev;

        left_c++;
        right_c--;
       }

   }
  while (left_c<=right_c);

  if (0<right_c)     QuickSortList(head,left->prev,left_c);
  if (count>left_c)  QuickSortList(left,tail,count-left_c);

 }

////////////////////////////////////
void QuickSortList2(cListNode* & head, cListNode* & tail, int count)
 {
  cListNode * temp = head;
  for (int i=0;i<count/2;i++) temp=temp->next;

  const int pivot = temp->number;
  

  cListNode * left = head;
  cListNode * right = tail;

  int left_c = 0;
  int right_c = count-1;

  do
   {
    while ( left->number < pivot ) left=left->next, left_c++;
    while ( (right_c>=left_c) && right->number > pivot ) right=right->prev, right_c--; 

    // exchanges the nodes themselves in list
    if (left_c<=right_c) 
       {
        cListNode *left_prev  = left->prev;
        cListNode *right_next = right->next;

        if (head==left) head=right;
        if (tail==right) tail=left;

        if (left->prev) left->prev->next = right;
        if (right->next) right->next->prev = left;

        if (left->next == right) 
             {
              left->prev  = right;
              right->next = left;
             }
        else { 
              right->next = left->next;
              left->prev  = right->prev;

              if (left->next) left->next->prev = right;
              if (right->prev) right->prev->next = left;
             }

        right->prev = left_prev;
        left->next  = right_next;

        swap(left,right);

        left  = left->next;
        right = right->prev;

        left_c++;
        right_c--;
       }
   }
  while (left_c<=right_c);

  if (0<right_c)    QuickSortList2(head,left->prev,left_c);
  if (count>left_c) QuickSortList2(left,tail,count-left_c);
 }


////////////////////////////////////
void main()
 {
  clock_t start, stop;

  cList z;

  InitList(z,10000);

  start=clock();
  while (start==clock()); // synchronizes to clock. 
  start=clock(); 

  QuickSortList(z.head,z.tail,z.count);

  stop=clock();

  // display results
  printf("dt = %0.2fs\n", (stop-start)/(double)CLOCKS_PER_SEC);


  //DisplayList(z);
 }
