// Mainframe.cpp

#include "stdafx.h"
#pragma hdrstop

#include "Mainframe.h"
#include "About.h"
#include "Constants.h"
#include "ConfigurationFactory.h"
#include "Configuration.h"
#include "Platform.h"
#include "Test.h"
#include "Constants.h"
#include "FixedFileInfo.h"
#include "Util.h"
#include <fstream>
#include <iomanip>

#pragma package(smart_init)
#pragma resource "*.dfm"

using namespace WaveSoftware;

const UINT ID_SYSMENU_ABOUT = 5000;

TfrmMainframe *frmMainframe;

__fastcall TfrmMainframe::TfrmMainframe(TComponent* Owner)
	: TForm(Owner)
{
	m_Configuration = NULL;

	HMENU sysmenu = ::GetSystemMenu(Handle, FALSE);

   ::AppendMenu(sysmenu,
   	MF_SEPARATOR,
      0,   /* ignored */
      NULL /* ignored */);

   ::AppendMenu(sysmenu,
   	MF_STRING,
      ID_SYSMENU_ABOUT,
      CSTR_SYSMENU_ABOUT.c_str());
}

_fastcall TfrmMainframe::~TfrmMainframe()
{
}

void __fastcall TfrmMainframe::OnSysCommand(TWMSysCommand& msg)
{
	if(ID_SYSMENU_ABOUT == msg.CmdType)
   {
   	dlgAbout->lblCopyright->Caption = m_Configuration->GetCopyright().c_str();
   	dlgAbout->ShowModal();

      msg.Result = 0;

      return;
   }

   TForm::Dispatch(&msg);
}
void __fastcall TfrmMainframe::actSaveExecute(TObject *Sender)
{
	if(!(svdMainframe->Execute()))
   	return;

   const Platform* platform = m_Configuration->GetPlatform();

   if(NULL == platform)
   	return;

   std::ofstream ofs(svdMainframe->FileName.c_str(), std::ios_base::out);

   FixedFileInfo ffi;

   ffi.SetPathname(Application->ExeName.c_str());

   ofs << CSTR_TEST_REPORT_TITLE << std::endl;
   ofs << CSTR_ECHECKLITE_EXE << CSTR_SPACE << CSTR_VERSION << CSTR_SPACE <<
   	ffi.GetFileVersion().ToString() <<
   	std::endl;
   ofs << CSTR_TEST_PLATFORM_COMPATIBILITY <<
   	Util::GetCurrentPlatformString() << std::endl;

   ofs << std::endl;

   ofs << CSTR_EQUALS_SEPARATORS << std::endl;

   ofs << std::setw(10) << std::left << CSTR_SEVERITY_TITLE <<
   	std::setw(16) << std::left <<	CSTR_VALUE_TITLE <<
      std::setw(53) << std::left << CSTR_DESCRIPTION_TITLE <<
      std::endl;

   ofs << CSTR_EQUALS_SEPARATORS << std::endl;

   const Tests& tests = platform->GetTests();
   Tests::const_iterator i = tests.begin();
   Tests::const_iterator j = tests.end();

   for(; i != j; i++)
   {
   	Test* test = *i;

      ofs << std::setw(10) << std::left;

      switch(test->GetStatus())
      {
         case Test::TS_OK:
            ofs << CSTR_OK;
            break;

         case Test::TS_LOW:
            ofs << CSTR_LOW;
            break;

         case Test::TS_MED:
            ofs << CSTR_MEDIUM;
            break;

         case Test::TS_HIGH:
            ofs << CSTR_HIGH;
            break;

         case Test::TS_UNKNOWN:
         default:
            ofs << CSTR_UNKNOWN;
            break;
      }

      ofs << std::setw(16) << std::left << test->GetStatusMsg().c_str();
      ofs << std::setw(53) << std::left << test->GetName().c_str() << std::endl;
   }

   ofs << CSTR_EQUALS_SEPARATORS << std::endl;
}

void __fastcall TfrmMainframe::actFinishExecute(TObject *Sender)
{
	Close();
}

void __fastcall TfrmMainframe::actDetailsUpdate(TObject *Sender)
{
	TAction* action = dynamic_cast<TAction*>(Sender);
   bool enabled = (lvTestResults->SelCount > 0);

   if(action->Enabled != enabled)
   	action->Enabled = enabled;
}

void __fastcall TfrmMainframe::FormCreate(TObject *Sender)
{
   std::string confile;

   if(1 == ParamCount())
   	confile = ParamStr(1).c_str();
   else if(2 == ParamCount())
       confile = ParamStr(2).c_str();

   // Project source contains error code for missmatched number of parameters.
   m_Configuration = ConfigurationFactory::Create(confile);

	// Initialize UI elements with the config file data.
   std::string tmp;

   tmp = CSTR_ECHECKLITE_LONG +
   	CSTR_SPACE + CSTR_HYPHEN + CSTR_SPACE +
      m_Configuration->GetTitle();

   Application->Title = tmp.c_str();
   Caption            = tmp.c_str();

   lblPlatform->Caption = Util::GetCurrentPlatformString().c_str();

   std::string testinfo = CSTR_TEST_INFO;
   TReplaceFlags rf;

   lblTestInfo->Caption = StringReplace(testinfo.c_str(),
   	CSTR_PROGRAM_TOKEN.c_str(),
   	m_Configuration->GetTitle().c_str(),
      rf);

   VerifyEnvironment();
}

void TfrmMainframe::VerifyEnvironment()
{
	if(NULL == m_Configuration)
   	return;

   Screen->Cursor = crHourGlass;

   lvTestResults->Items->BeginUpdate();
   lvTestResults->Items->Clear();

   const Platform* platform = m_Configuration->GetPlatform();

   if(NULL != platform)
   {
   	platform->Verify();

      // We record the total number of incompatibilities and store
      // it in the registry for downstream tools; for example, installation
      // programs.
      int testscores[] =
      {
      	0, /* UNKNOWN */
         0, /* OK */
         0, /* LOW */
         0, /* MED */
         0  /* HIGH */
      };

      const Tests& tests = platform->GetTests();

      Tests::const_iterator i = tests.begin();
      Tests::const_iterator j = tests.end();

      for(; i != j; i++)
      {
      	Test* test = *i;
         Test::TEST_STATUS status = test->GetStatus();
         TListItem* item = lvTestResults->Items->Add();

         ++testscores[(int) status];

         switch(status)
         {
         	case Test::TS_OK:
            	item->Caption = CSTR_OK.c_str();
               break;

            case Test::TS_LOW:
            	item->Caption = CSTR_LOW.c_str();
               break;

            case Test::TS_MED:
            	item->Caption = CSTR_MEDIUM.c_str();
               break;

            case Test::TS_HIGH:
            	item->Caption = CSTR_HIGH.c_str();
               break;

            case Test::TS_UNKNOWN:
            default:
            	item->Caption = CSTR_UNKNOWN.c_str();
               break;
         }

         item->ImageIndex = (int) status;
         item->SubItems->Add(test->GetStatusMsg().c_str());
         item->SubItems->Add(test->GetName().c_str());
      }

      // Store the incompatibilities in the registry.
      TRegistry* reg = new TRegistry;

      reg->RootKey = HKEY_LOCAL_MACHINE;

      if(reg->OpenKey(m_Configuration->GetRegistryKey().c_str(), true))
      {
      	reg->WriteInteger(CSTR_UNKNOWN.c_str(), testscores[0]);
         reg->WriteInteger(CSTR_OK.c_str(), testscores[1]);
         reg->WriteInteger(CSTR_LOW.c_str(), testscores[2]);
         reg->WriteInteger(CSTR_MEDIUM.c_str(), testscores[3]);
         reg->WriteInteger(CSTR_HIGH.c_str(), testscores[4]);

         reg->CloseKey();
      }

      delete reg;  reg = NULL;
   }

   lvTestResults->Items->EndUpdate();
   Screen->Cursor = crDefault;
}

void __fastcall TfrmMainframe::actSaveUpdate(TObject *Sender)
{
	TAction* action = dynamic_cast<TAction*>(Sender);
   bool enabled = (lvTestResults->Items->Count > 0);

	if(action->Enabled != enabled)
   	action->Enabled = enabled;
}

void __fastcall TfrmMainframe::actDetailsExecute(TObject *Sender)
{
	std::string dir = m_Configuration->GetDetailsRoot();
   std::string filename = m_Configuration->GetName();
   std::string pathname = CSTR_FILE_URL +
   	ExtractFilePath(Application->ExeName).c_str() + dir + CSTR_BACKSLASH +
      filename + CSTR_HTM_SUFFIX;

   Util::ShellExecute(pathname.c_str(), NULL);
}

void __fastcall TfrmMainframe::FormDestroy(TObject *Sender)
{
	if(NULL != m_Configuration)
   {
   	delete m_Configuration;  m_Configuration = NULL;
   }
}


