// SystemFile.cpp

#include "stdafx.h"
#pragma hdrstop

#include "SystemFile.h"
#include "FixedFileInfo.h"
#include "Util.h"
#include "Constants.h"

namespace WaveSoftware
{

   SystemFile::SystemFile()
   {
   }

   SystemFile::~SystemFile()
   {
      Util::DestroyVersions(m_Versions);
   }

   bool SystemFile::Initialize(const StringMap& data)
   {
      if(!(Test::Initialize(data)))
         return(false);

      if(!(Util::GetFromMap(data, CSTR_FILENAME, m_Filename)))
         return(false);

      std::string tmp;

      if(!(Util::GetFromMap(data, CSTR_VERSIONS, tmp)))
         return(false);

      if(!(Util::GetFromString(tmp, m_Versions)))
         return(false);

      return(true);
   }

   bool SystemFile::Verify()
   {
      SetStatus(TS_HIGH);
      SetStatusMsg(CSTR_UNDETECTED);

      FixedFileInfo ffi;

      if(!(IsInSystemDir(&ffi)))
         if(!(IsInWindowsDir(&ffi)))
            if(!(IsInPathDir(&ffi)))
               return(true);

      const Version& v = ffi.GetFileVersion();
      int cr = Util::CompareVersionWithVersions(v, m_Versions);

      SetStatus((Test::TEST_STATUS) cr);
      SetStatusMsg(v.ToString());
      
      return(true);
   }

   bool SystemFile::IsInSystemDir(FixedFileInfo* ffi)
   {
      TCHAR buffer[_MAX_PATH];
      UINT r = 0;

      r = ::GetSystemDirectory(buffer, sizeof(buffer));

      if(r > 0)
      {
         _tcscat(buffer, CSTR_BACKSLASH.c_str());
         _tcscat(buffer, m_Filename.c_str());

         ffi->SetPathname(buffer);

         return(ffi->IsAvailable());
      }
                 
      return(false);
   }

   bool SystemFile::IsInWindowsDir(FixedFileInfo* ffi)
   {
      TCHAR buffer[_MAX_PATH];
      UINT r = 0;

      r = ::GetWindowsDirectory(buffer, sizeof(buffer));

      if(r > 0)
      {
         _tcscat(buffer, CSTR_BACKSLASH.c_str());
         _tcscat(buffer, m_Filename.c_str());

         ffi->SetPathname(buffer);

         return(ffi->IsAvailable());
      }

      return(false);
   }

   bool SystemFile::IsInPathDir(FixedFileInfo* ffi)
   {
      LPVOID env = ::GetEnvironmentStrings();
      TCHAR buffer[0x1000];
      DWORD s = 0;

      s = ::GetEnvironmentVariable(CSTR_PATH.c_str(), buffer, sizeof(buffer));

      if(s > 0)
      {
         // The PATH environment variable is a tokenized string with
         // the ';' character as the delimiter.
         TCHAR* token = _tcstok(buffer, CSTR_SEMICOLON.c_str());

         while(NULL != token)
         {
            TCHAR path[_MAX_PATH];

            _tcscpy(path, token);

            // Is there already a directory delimiter?
            if('\\' != path[_tcslen(path) - 1])
               _tcscat(path, CSTR_BACKSLASH.c_str());

            ffi->SetPathname(path);

            if(ffi->IsAvailable())
               break;

            token = _tcstok(NULL, CSTR_SEMICOLON.c_str());
         }
      }

      ::FreeEnvironmentStrings((TCHAR*) env);

      return(ffi->IsAvailable());
   }

} // end of namespace
