// TestFactory.cpp

#include "stdafx.h"
#pragma hdrstop

#include "TestFactory.h"
#include "Tests.h"
#include "Util.h"
#include "Constants.h"

namespace WaveSoftware
{
   Test* CreateNull(const StringMap& data);
   Test* CreateRam(const StringMap& data);
   Test* CreateCpuSpeed(const StringMap& data);
   Test* CreateFreeDiskSpace(const StringMap& data);
   Test* CreateNetworkCard(const StringMap& data);
   Test* CreateServicePack(const StringMap& data);
   Test* CreateMDAC(const StringMap& data);
   Test* CreateRegistryFile(const StringMap& data);
   Test* CreateSystemFile(const StringMap& data);

   typedef Test* (*PFN_CREATETEST)(const StringMap& data);

   typedef struct tagTESTENTRY
   {
      int type;
      PFN_CREATETEST func;
   } TESTENTRY;

   TESTENTRY g_Tests[] =
   {
      { 0, CreateNull },
      { 1, CreateRam },
      { 2, CreateCpuSpeed },
      { 3, CreateFreeDiskSpace },
      { 4, CreateNetworkCard },
      { 5, CreateServicePack },
      { 6, CreateMDAC },
      { 7, CreateRegistryFile },
      { 8, CreateSystemFile }
   };
   
   Test* TestFactory::Create(const StringMap& data)
   {
      int type = 0;

      if(!(Util::GetFromMap(data, CSTR_TYPE, type)))
         return(NULL);

      // Find and execute the appropriate test factory function.
      Test* test = NULL;

      for(int i = 0; i < sizeof(g_Tests)/ sizeof(g_Tests[0]); i++)
      {
         if(type == g_Tests[i].type)
         {
            test = (g_Tests[i].func)(data);
            break;
         }
      }

      return(test);
   }

   Test* CreateNull(const StringMap& data)
   {
      return(NULL);
   }

   Test* CreateRam(const StringMap& data)
   {
      Ram* t = new Ram;

      if(!(t->Initialize(data)))
      {
         delete t;  t = NULL;
      }

      return(t);
   }

   Test* CreateCpuSpeed(const StringMap& data)
   {
      CpuSpeed* t = new CpuSpeed;

      if(!(t->Initialize(data)))
      {
         delete t;  t = NULL;
      }

      return(t);
   }
   
   Test* CreateFreeDiskSpace(const StringMap& data)
   {
      FreeDiskSpace* t = new FreeDiskSpace;

      if(!(t->Initialize(data)))
      {
         delete t;  t = NULL;
      }

      return(t);
   }
   
   Test* CreateNetworkCard(const StringMap& data)
   {
      NetworkCard* t = new NetworkCard;

      if(!(t->Initialize(data)))
      {
         delete t;  t = NULL;
      }

      return(t);
   }
   
   Test* CreateServicePack(const StringMap& data)
   {
      ServicePack* t = new ServicePack;

      if(!(t->Initialize(data)))
      {
         delete t;  t = NULL;
      }

      return(t);
   }
   
   Test* CreateMDAC(const StringMap& data)
   {
      MDAC* t = new MDAC;

      if(!(t->Initialize(data)))
      {
         delete t;  t = NULL;
      }

      return(t);
   }
   
   Test* CreateRegistryFile(const StringMap& data)
   {
      RegistryFile* t = new RegistryFile;

      if(!(t->Initialize(data)))
      {
         delete t;  t = NULL;
      }

      return(t);
   }
   
   Test* CreateSystemFile(const StringMap& data)
   {
      SystemFile* t = new SystemFile;

      if(!(t->Initialize(data)))
      {
         delete t;  t = NULL;
      }

      return(t);
   }
   
} // end of namespace
